package com.novaordis.thread194629.distrib;

import org.infinispan.config.Configuration;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.configuration.cache.LegacyConfigurationAdaptor;
import org.infinispan.configuration.global.LegacyGlobalConfigurationAdaptor;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.util.concurrent.IsolationLevel;

public class CacheManagerHolder {

	private static final CacheManagerHolder instance = new CacheManagerHolder();
	private  EmbeddedCacheManager cm;
	public static final String MSG_KEY = "isDone";
	public static final String MSG_DONE = "Done";
	public static final String MSG_WORKING = "Working";
	public static final String CACHE_MON = "TEST-CACHE";
	public static final String CACHE_SRC = "SRC-CACHE";
	public static final String CACHE_DEST = "DES-CACHE";
	
	private CacheManagerHolder(){
		try {
			cm = createAndConfigureCacheManager();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		cm.startCaches(CACHE_MON,CACHE_SRC,CACHE_DEST);
	}
	
	public static CacheManagerHolder getInstance(){
		return instance;
	}
	
	public EmbeddedCacheManager getCacheManager(){
		return cm;
	}
	
	private EmbeddedCacheManager createAndConfigureCacheManager() throws Exception
    {
        GlobalConfiguration gc = GlobalConfiguration.getClusteredDefault();

        gc.setClusterName("TEST");

        Configuration lc = new Configuration();
        lc.setCacheMode(Configuration.CacheMode.DIST_ASYNC);
//        lc.setCacheMode(Configuration.CacheMode.DIST_SYNC);
        lc.setRehashEnabled(true);
        lc.setSyncReplTimeout(300000L);
        lc.setLockAcquisitionTimeout(300000L);
        lc.setEnableDeadlockDetection(true);
        lc.setInvocationBatchingEnabled(true);
        lc.setEagerLockSingleNode(false);
        lc.setExpirationLifespan(-1);
        lc.setConcurrencyLevel(2000);
        lc.setIsolationLevel(IsolationLevel.READ_COMMITTED);
        lc.setSyncCommitPhase(true);
        lc.setSyncRollbackPhase(true);
        lc.setNumOwners(2);
        

//        return new DefaultCacheManager(
//            LegacyGlobalConfigurationAdaptor.adapt(gc),
//            LegacyConfigurationAdaptor.adapt(lc), false);
        return new DefaultCacheManager(
                gc,lc, true);
    }

}
