package com.novaordis.thread194629.distrib;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.manager.EmbeddedCacheManager;
import org.junit.Assert;
import org.junit.Test;

/**
 * @author <a href="mailto:ovidiu@novaordis.com">Ovidiu Feodorov</a>
 *
 * Copyright 2012 Nova Ordis LLC
 */
public class ConcurrentWriteReadDistribTestSlaveTest extends Assert
{
    // Constants -----------------------------------------------------------------------------------

    private static final Logger log = Logger.getLogger(ConcurrentWriteReadDistribTestSlaveTest.class);
    private static final String SOURCE_CHILD_NODE_PREFIX         = "ChildNode";

    // Static --------------------------------------------------------------------------------------

//    public static void main(String [] args)
////    {
 //     Thread t = new Thread(new ConcurrentWriteReadDistribTestSlave());
//      t.start();
//    }

    public void testReadsOnSlaveOnceDestCacheIsLoadedByMaster() throws Exception
    {
        final EmbeddedCacheManager cm = CacheManagerHolder.getInstance().getCacheManager();

        Cache<Object, Object> c = cm.getCache(CacheManagerHolder.CACHE_MON);
        Cache<Object, Object> sc = cm.getCache(CacheManagerHolder.CACHE_SRC);
        Cache<Object, Object> dc = cm.getCache(CacheManagerHolder.CACHE_DEST);

        // no need to start() the cache, creation implicitly starts
        waitTillCacheIsInitialized(c);
        waitTillCacheIsInitialized(sc);
        waitTillCacheIsInitialized(dc);

        final List<String> pathToProcess = getPathsToProcessInSource();
        
        waitTillAllKeysArePopulatedInDestinationCache(c);
        
        checkIfDestinationCacheHasAllKeys(dc, pathToProcess);
    }

	private void checkIfDestinationCacheHasAllKeys(Cache<Object, Object> dc,
			final List<String> pathToProcess) throws Exception {
		for (final String sourceChildPath : pathToProcess)
        {
          final String key = sourceChildPath;

          Object value = dc.get(key);
          

          if (value == null)
          {
            throw new Exception(dc.getName()+" Destination cache lacked a key for "
                + key);
          }

          if (!value.equals(key))
          {
            throw new Exception(
                dc.getName()+" Destination cache had the wrong value for key "
                    + key + ", value was " + value.toString());
          }
          
          log.info(dc.getName()+" Found good key " + key + " in destination root node ");
        }
	}

	private void waitTillAllKeysArePopulatedInDestinationCache(
			Cache<Object, Object> c) throws InterruptedException {
		int retries = 125;
        int count = 0;
        String state; 
        while(true){
        	count++;
            state = (String)c.get(CacheManagerHolder.MSG_KEY);
            if(null!= state && state.equals(CacheManagerHolder.MSG_DONE)){
            	break;
            }  
            if(count >= retries){
            	break;
            }
            else{
            	Thread.sleep(100);
            }
        }
        log.info("State = " +state +" achieved after " + count*100/1000 +" seconds");
	}

	private List<String> getPathsToProcessInSource() {
		final List<String> pathToProcess = new ArrayList<String>();
        for (int i = 0; i < ConcurrentWriteReadDistribTestMasterTest.CACHE_ENTRY_COUNT; i++)
        {
          final String childNodeName = SOURCE_CHILD_NODE_PREFIX
              + Integer.toString(i);
          pathToProcess.add(childNodeName);
        }
		return pathToProcess;
	}

	private void waitTillCacheIsInitialized(Cache<Object, Object> sc)
			throws InterruptedException {
		while (!sc.getCacheManager().getStatus().allowInvocations())
        {
            log.info("sleeping, waiting for the cache to start ...");
            Thread.sleep(500l);
        }
	}

    @Test
    public void run() throws Exception
    {
       System.out.println("Running slave.");
//      while (true) {
//        if (key.startsWith("start")) {
          try
          {
            Thread.sleep(15000);
          } catch (InterruptedException e)
          {
            e.printStackTrace();
          }
          try {
        	CacheManagerHolder.getInstance();  
			testReadsOnSlaveOnceDestCacheIsLoadedByMaster();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}
//        }
//      }
    }
}
